# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from . import outputs
from ._inputs import *

__all__ = ['RegionSecurityPolicyRuleArgs', 'RegionSecurityPolicyRule']

@pulumi.input_type
class RegionSecurityPolicyRuleArgs:
    def __init__(__self__, *,
                 action: pulumi.Input[str],
                 priority: pulumi.Input[int],
                 region: pulumi.Input[str],
                 security_policy: pulumi.Input[str],
                 description: Optional[pulumi.Input[str]] = None,
                 match: Optional[pulumi.Input['RegionSecurityPolicyRuleMatchArgs']] = None,
                 network_match: Optional[pulumi.Input['RegionSecurityPolicyRuleNetworkMatchArgs']] = None,
                 preview: Optional[pulumi.Input[bool]] = None,
                 project: Optional[pulumi.Input[str]] = None):
        """
        The set of arguments for constructing a RegionSecurityPolicyRule resource.
        :param pulumi.Input[str] action: The Action to perform when the rule is matched. The following are the valid actions:
               * allow: allow access to target.
               * deny(STATUS): deny access to target, returns the HTTP response code specified. Valid values for STATUS are 403, 404, and 502.
               * rate_based_ban: limit client traffic to the configured threshold and ban the client if the traffic exceeds the threshold. Configure parameters for this action in RateLimitOptions. Requires rateLimitOptions to be set.
               * redirect: redirect to a different target. This can either be an internal reCAPTCHA redirect, or an external URL-based redirect via a 302 response. Parameters for this action can be configured via redirectOptions. This action is only supported in Global Security Policies of type CLOUD_ARMOR.
               * throttle: limit client traffic to the configured threshold. Configure parameters for this action in rateLimitOptions. Requires rateLimitOptions to be set for this.
        :param pulumi.Input[int] priority: An integer indicating the priority of a rule in the list.
               The priority must be a positive value between 0 and 2147483647.
               Rules are evaluated from highest to lowest priority where 0 is the highest priority and 2147483647 is the lowest priority.
        :param pulumi.Input[str] region: The Region in which the created Region Security Policy rule should reside.
        :param pulumi.Input[str] security_policy: The name of the security policy this rule belongs to.
               
               
               - - -
        :param pulumi.Input[str] description: An optional description of this resource. Provide this property when you create the resource.
        :param pulumi.Input['RegionSecurityPolicyRuleMatchArgs'] match: A match condition that incoming traffic is evaluated against.
               If it evaluates to true, the corresponding 'action' is enforced.
               Structure is documented below.
        :param pulumi.Input['RegionSecurityPolicyRuleNetworkMatchArgs'] network_match: A match condition that incoming packets are evaluated against for CLOUD_ARMOR_NETWORK security policies. If it matches, the corresponding 'action' is enforced.
               The match criteria for a rule consists of built-in match fields (like 'srcIpRanges') and potentially multiple user-defined match fields ('userDefinedFields').
               Field values may be extracted directly from the packet or derived from it (e.g. 'srcRegionCodes'). Some fields may not be present in every packet (e.g. 'srcPorts'). A user-defined field is only present if the base header is found in the packet and the entire field is in bounds.
               Each match field may specify which values can match it, listing one or more ranges, prefixes, or exact values that are considered a match for the field. A field value must be present in order to match a specified match field. If no match values are specified for a match field, then any field value is considered to match it, and it's not required to be present. For strings specifying '*' is also equivalent to match all.
               For a packet to match a rule, all specified match fields must match the corresponding field values derived from the packet.
               Example:
               networkMatch: srcIpRanges: - "192.0.2.0/24" - "198.51.100.0/24" userDefinedFields: - name: "ipv4_fragment_offset" values: - "1-0x1fff"
               The above match condition matches packets with a source IP in 192.0.2.0/24 or 198.51.100.0/24 and a user-defined field named "ipv4_fragment_offset" with a value between 1 and 0x1fff inclusive
               Structure is documented below.
        :param pulumi.Input[bool] preview: If set to true, the specified action is not enforced.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        """
        pulumi.set(__self__, "action", action)
        pulumi.set(__self__, "priority", priority)
        pulumi.set(__self__, "region", region)
        pulumi.set(__self__, "security_policy", security_policy)
        if description is not None:
            pulumi.set(__self__, "description", description)
        if match is not None:
            pulumi.set(__self__, "match", match)
        if network_match is not None:
            pulumi.set(__self__, "network_match", network_match)
        if preview is not None:
            pulumi.set(__self__, "preview", preview)
        if project is not None:
            pulumi.set(__self__, "project", project)

    @property
    @pulumi.getter
    def action(self) -> pulumi.Input[str]:
        """
        The Action to perform when the rule is matched. The following are the valid actions:
        * allow: allow access to target.
        * deny(STATUS): deny access to target, returns the HTTP response code specified. Valid values for STATUS are 403, 404, and 502.
        * rate_based_ban: limit client traffic to the configured threshold and ban the client if the traffic exceeds the threshold. Configure parameters for this action in RateLimitOptions. Requires rateLimitOptions to be set.
        * redirect: redirect to a different target. This can either be an internal reCAPTCHA redirect, or an external URL-based redirect via a 302 response. Parameters for this action can be configured via redirectOptions. This action is only supported in Global Security Policies of type CLOUD_ARMOR.
        * throttle: limit client traffic to the configured threshold. Configure parameters for this action in rateLimitOptions. Requires rateLimitOptions to be set for this.
        """
        return pulumi.get(self, "action")

    @action.setter
    def action(self, value: pulumi.Input[str]):
        pulumi.set(self, "action", value)

    @property
    @pulumi.getter
    def priority(self) -> pulumi.Input[int]:
        """
        An integer indicating the priority of a rule in the list.
        The priority must be a positive value between 0 and 2147483647.
        Rules are evaluated from highest to lowest priority where 0 is the highest priority and 2147483647 is the lowest priority.
        """
        return pulumi.get(self, "priority")

    @priority.setter
    def priority(self, value: pulumi.Input[int]):
        pulumi.set(self, "priority", value)

    @property
    @pulumi.getter
    def region(self) -> pulumi.Input[str]:
        """
        The Region in which the created Region Security Policy rule should reside.
        """
        return pulumi.get(self, "region")

    @region.setter
    def region(self, value: pulumi.Input[str]):
        pulumi.set(self, "region", value)

    @property
    @pulumi.getter(name="securityPolicy")
    def security_policy(self) -> pulumi.Input[str]:
        """
        The name of the security policy this rule belongs to.


        - - -
        """
        return pulumi.get(self, "security_policy")

    @security_policy.setter
    def security_policy(self, value: pulumi.Input[str]):
        pulumi.set(self, "security_policy", value)

    @property
    @pulumi.getter
    def description(self) -> Optional[pulumi.Input[str]]:
        """
        An optional description of this resource. Provide this property when you create the resource.
        """
        return pulumi.get(self, "description")

    @description.setter
    def description(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "description", value)

    @property
    @pulumi.getter
    def match(self) -> Optional[pulumi.Input['RegionSecurityPolicyRuleMatchArgs']]:
        """
        A match condition that incoming traffic is evaluated against.
        If it evaluates to true, the corresponding 'action' is enforced.
        Structure is documented below.
        """
        return pulumi.get(self, "match")

    @match.setter
    def match(self, value: Optional[pulumi.Input['RegionSecurityPolicyRuleMatchArgs']]):
        pulumi.set(self, "match", value)

    @property
    @pulumi.getter(name="networkMatch")
    def network_match(self) -> Optional[pulumi.Input['RegionSecurityPolicyRuleNetworkMatchArgs']]:
        """
        A match condition that incoming packets are evaluated against for CLOUD_ARMOR_NETWORK security policies. If it matches, the corresponding 'action' is enforced.
        The match criteria for a rule consists of built-in match fields (like 'srcIpRanges') and potentially multiple user-defined match fields ('userDefinedFields').
        Field values may be extracted directly from the packet or derived from it (e.g. 'srcRegionCodes'). Some fields may not be present in every packet (e.g. 'srcPorts'). A user-defined field is only present if the base header is found in the packet and the entire field is in bounds.
        Each match field may specify which values can match it, listing one or more ranges, prefixes, or exact values that are considered a match for the field. A field value must be present in order to match a specified match field. If no match values are specified for a match field, then any field value is considered to match it, and it's not required to be present. For strings specifying '*' is also equivalent to match all.
        For a packet to match a rule, all specified match fields must match the corresponding field values derived from the packet.
        Example:
        networkMatch: srcIpRanges: - "192.0.2.0/24" - "198.51.100.0/24" userDefinedFields: - name: "ipv4_fragment_offset" values: - "1-0x1fff"
        The above match condition matches packets with a source IP in 192.0.2.0/24 or 198.51.100.0/24 and a user-defined field named "ipv4_fragment_offset" with a value between 1 and 0x1fff inclusive
        Structure is documented below.
        """
        return pulumi.get(self, "network_match")

    @network_match.setter
    def network_match(self, value: Optional[pulumi.Input['RegionSecurityPolicyRuleNetworkMatchArgs']]):
        pulumi.set(self, "network_match", value)

    @property
    @pulumi.getter
    def preview(self) -> Optional[pulumi.Input[bool]]:
        """
        If set to true, the specified action is not enforced.
        """
        return pulumi.get(self, "preview")

    @preview.setter
    def preview(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "preview", value)

    @property
    @pulumi.getter
    def project(self) -> Optional[pulumi.Input[str]]:
        """
        The ID of the project in which the resource belongs.
        If it is not provided, the provider project is used.
        """
        return pulumi.get(self, "project")

    @project.setter
    def project(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "project", value)


@pulumi.input_type
class _RegionSecurityPolicyRuleState:
    def __init__(__self__, *,
                 action: Optional[pulumi.Input[str]] = None,
                 description: Optional[pulumi.Input[str]] = None,
                 match: Optional[pulumi.Input['RegionSecurityPolicyRuleMatchArgs']] = None,
                 network_match: Optional[pulumi.Input['RegionSecurityPolicyRuleNetworkMatchArgs']] = None,
                 preview: Optional[pulumi.Input[bool]] = None,
                 priority: Optional[pulumi.Input[int]] = None,
                 project: Optional[pulumi.Input[str]] = None,
                 region: Optional[pulumi.Input[str]] = None,
                 security_policy: Optional[pulumi.Input[str]] = None):
        """
        Input properties used for looking up and filtering RegionSecurityPolicyRule resources.
        :param pulumi.Input[str] action: The Action to perform when the rule is matched. The following are the valid actions:
               * allow: allow access to target.
               * deny(STATUS): deny access to target, returns the HTTP response code specified. Valid values for STATUS are 403, 404, and 502.
               * rate_based_ban: limit client traffic to the configured threshold and ban the client if the traffic exceeds the threshold. Configure parameters for this action in RateLimitOptions. Requires rateLimitOptions to be set.
               * redirect: redirect to a different target. This can either be an internal reCAPTCHA redirect, or an external URL-based redirect via a 302 response. Parameters for this action can be configured via redirectOptions. This action is only supported in Global Security Policies of type CLOUD_ARMOR.
               * throttle: limit client traffic to the configured threshold. Configure parameters for this action in rateLimitOptions. Requires rateLimitOptions to be set for this.
        :param pulumi.Input[str] description: An optional description of this resource. Provide this property when you create the resource.
        :param pulumi.Input['RegionSecurityPolicyRuleMatchArgs'] match: A match condition that incoming traffic is evaluated against.
               If it evaluates to true, the corresponding 'action' is enforced.
               Structure is documented below.
        :param pulumi.Input['RegionSecurityPolicyRuleNetworkMatchArgs'] network_match: A match condition that incoming packets are evaluated against for CLOUD_ARMOR_NETWORK security policies. If it matches, the corresponding 'action' is enforced.
               The match criteria for a rule consists of built-in match fields (like 'srcIpRanges') and potentially multiple user-defined match fields ('userDefinedFields').
               Field values may be extracted directly from the packet or derived from it (e.g. 'srcRegionCodes'). Some fields may not be present in every packet (e.g. 'srcPorts'). A user-defined field is only present if the base header is found in the packet and the entire field is in bounds.
               Each match field may specify which values can match it, listing one or more ranges, prefixes, or exact values that are considered a match for the field. A field value must be present in order to match a specified match field. If no match values are specified for a match field, then any field value is considered to match it, and it's not required to be present. For strings specifying '*' is also equivalent to match all.
               For a packet to match a rule, all specified match fields must match the corresponding field values derived from the packet.
               Example:
               networkMatch: srcIpRanges: - "192.0.2.0/24" - "198.51.100.0/24" userDefinedFields: - name: "ipv4_fragment_offset" values: - "1-0x1fff"
               The above match condition matches packets with a source IP in 192.0.2.0/24 or 198.51.100.0/24 and a user-defined field named "ipv4_fragment_offset" with a value between 1 and 0x1fff inclusive
               Structure is documented below.
        :param pulumi.Input[bool] preview: If set to true, the specified action is not enforced.
        :param pulumi.Input[int] priority: An integer indicating the priority of a rule in the list.
               The priority must be a positive value between 0 and 2147483647.
               Rules are evaluated from highest to lowest priority where 0 is the highest priority and 2147483647 is the lowest priority.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[str] region: The Region in which the created Region Security Policy rule should reside.
        :param pulumi.Input[str] security_policy: The name of the security policy this rule belongs to.
               
               
               - - -
        """
        if action is not None:
            pulumi.set(__self__, "action", action)
        if description is not None:
            pulumi.set(__self__, "description", description)
        if match is not None:
            pulumi.set(__self__, "match", match)
        if network_match is not None:
            pulumi.set(__self__, "network_match", network_match)
        if preview is not None:
            pulumi.set(__self__, "preview", preview)
        if priority is not None:
            pulumi.set(__self__, "priority", priority)
        if project is not None:
            pulumi.set(__self__, "project", project)
        if region is not None:
            pulumi.set(__self__, "region", region)
        if security_policy is not None:
            pulumi.set(__self__, "security_policy", security_policy)

    @property
    @pulumi.getter
    def action(self) -> Optional[pulumi.Input[str]]:
        """
        The Action to perform when the rule is matched. The following are the valid actions:
        * allow: allow access to target.
        * deny(STATUS): deny access to target, returns the HTTP response code specified. Valid values for STATUS are 403, 404, and 502.
        * rate_based_ban: limit client traffic to the configured threshold and ban the client if the traffic exceeds the threshold. Configure parameters for this action in RateLimitOptions. Requires rateLimitOptions to be set.
        * redirect: redirect to a different target. This can either be an internal reCAPTCHA redirect, or an external URL-based redirect via a 302 response. Parameters for this action can be configured via redirectOptions. This action is only supported in Global Security Policies of type CLOUD_ARMOR.
        * throttle: limit client traffic to the configured threshold. Configure parameters for this action in rateLimitOptions. Requires rateLimitOptions to be set for this.
        """
        return pulumi.get(self, "action")

    @action.setter
    def action(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "action", value)

    @property
    @pulumi.getter
    def description(self) -> Optional[pulumi.Input[str]]:
        """
        An optional description of this resource. Provide this property when you create the resource.
        """
        return pulumi.get(self, "description")

    @description.setter
    def description(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "description", value)

    @property
    @pulumi.getter
    def match(self) -> Optional[pulumi.Input['RegionSecurityPolicyRuleMatchArgs']]:
        """
        A match condition that incoming traffic is evaluated against.
        If it evaluates to true, the corresponding 'action' is enforced.
        Structure is documented below.
        """
        return pulumi.get(self, "match")

    @match.setter
    def match(self, value: Optional[pulumi.Input['RegionSecurityPolicyRuleMatchArgs']]):
        pulumi.set(self, "match", value)

    @property
    @pulumi.getter(name="networkMatch")
    def network_match(self) -> Optional[pulumi.Input['RegionSecurityPolicyRuleNetworkMatchArgs']]:
        """
        A match condition that incoming packets are evaluated against for CLOUD_ARMOR_NETWORK security policies. If it matches, the corresponding 'action' is enforced.
        The match criteria for a rule consists of built-in match fields (like 'srcIpRanges') and potentially multiple user-defined match fields ('userDefinedFields').
        Field values may be extracted directly from the packet or derived from it (e.g. 'srcRegionCodes'). Some fields may not be present in every packet (e.g. 'srcPorts'). A user-defined field is only present if the base header is found in the packet and the entire field is in bounds.
        Each match field may specify which values can match it, listing one or more ranges, prefixes, or exact values that are considered a match for the field. A field value must be present in order to match a specified match field. If no match values are specified for a match field, then any field value is considered to match it, and it's not required to be present. For strings specifying '*' is also equivalent to match all.
        For a packet to match a rule, all specified match fields must match the corresponding field values derived from the packet.
        Example:
        networkMatch: srcIpRanges: - "192.0.2.0/24" - "198.51.100.0/24" userDefinedFields: - name: "ipv4_fragment_offset" values: - "1-0x1fff"
        The above match condition matches packets with a source IP in 192.0.2.0/24 or 198.51.100.0/24 and a user-defined field named "ipv4_fragment_offset" with a value between 1 and 0x1fff inclusive
        Structure is documented below.
        """
        return pulumi.get(self, "network_match")

    @network_match.setter
    def network_match(self, value: Optional[pulumi.Input['RegionSecurityPolicyRuleNetworkMatchArgs']]):
        pulumi.set(self, "network_match", value)

    @property
    @pulumi.getter
    def preview(self) -> Optional[pulumi.Input[bool]]:
        """
        If set to true, the specified action is not enforced.
        """
        return pulumi.get(self, "preview")

    @preview.setter
    def preview(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "preview", value)

    @property
    @pulumi.getter
    def priority(self) -> Optional[pulumi.Input[int]]:
        """
        An integer indicating the priority of a rule in the list.
        The priority must be a positive value between 0 and 2147483647.
        Rules are evaluated from highest to lowest priority where 0 is the highest priority and 2147483647 is the lowest priority.
        """
        return pulumi.get(self, "priority")

    @priority.setter
    def priority(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "priority", value)

    @property
    @pulumi.getter
    def project(self) -> Optional[pulumi.Input[str]]:
        """
        The ID of the project in which the resource belongs.
        If it is not provided, the provider project is used.
        """
        return pulumi.get(self, "project")

    @project.setter
    def project(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "project", value)

    @property
    @pulumi.getter
    def region(self) -> Optional[pulumi.Input[str]]:
        """
        The Region in which the created Region Security Policy rule should reside.
        """
        return pulumi.get(self, "region")

    @region.setter
    def region(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "region", value)

    @property
    @pulumi.getter(name="securityPolicy")
    def security_policy(self) -> Optional[pulumi.Input[str]]:
        """
        The name of the security policy this rule belongs to.


        - - -
        """
        return pulumi.get(self, "security_policy")

    @security_policy.setter
    def security_policy(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "security_policy", value)


class RegionSecurityPolicyRule(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 action: Optional[pulumi.Input[str]] = None,
                 description: Optional[pulumi.Input[str]] = None,
                 match: Optional[pulumi.Input[pulumi.InputType['RegionSecurityPolicyRuleMatchArgs']]] = None,
                 network_match: Optional[pulumi.Input[pulumi.InputType['RegionSecurityPolicyRuleNetworkMatchArgs']]] = None,
                 preview: Optional[pulumi.Input[bool]] = None,
                 priority: Optional[pulumi.Input[int]] = None,
                 project: Optional[pulumi.Input[str]] = None,
                 region: Optional[pulumi.Input[str]] = None,
                 security_policy: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        """
        ## Example Usage
        ### Region Security Policy Rule Basic

        ```python
        import pulumi
        import pulumi_gcp as gcp

        default = gcp.compute.RegionSecurityPolicy("default",
            region="us-west2",
            description="basic region security policy",
            type="CLOUD_ARMOR",
            opts=pulumi.ResourceOptions(provider=google_beta))
        policy_rule = gcp.compute.RegionSecurityPolicyRule("policyRule",
            region="us-west2",
            security_policy=default.name,
            description="new rule",
            priority=100,
            match=gcp.compute.RegionSecurityPolicyRuleMatchArgs(
                versioned_expr="SRC_IPS_V1",
                config=gcp.compute.RegionSecurityPolicyRuleMatchConfigArgs(
                    src_ip_ranges=["10.10.0.0/16"],
                ),
            ),
            action="allow",
            preview=True,
            opts=pulumi.ResourceOptions(provider=google_beta))
        ```
        ### Region Security Policy Rule Multiple Rules

        ```python
        import pulumi
        import pulumi_gcp as gcp

        default = gcp.compute.RegionSecurityPolicy("default",
            region="us-west2",
            description="basic region security policy",
            type="CLOUD_ARMOR",
            opts=pulumi.ResourceOptions(provider=google_beta))
        policy_rule_one = gcp.compute.RegionSecurityPolicyRule("policyRuleOne",
            region="us-west2",
            security_policy=default.name,
            description="new rule one",
            priority=100,
            match=gcp.compute.RegionSecurityPolicyRuleMatchArgs(
                versioned_expr="SRC_IPS_V1",
                config=gcp.compute.RegionSecurityPolicyRuleMatchConfigArgs(
                    src_ip_ranges=["10.10.0.0/16"],
                ),
            ),
            action="allow",
            preview=True,
            opts=pulumi.ResourceOptions(provider=google_beta))
        policy_rule_two = gcp.compute.RegionSecurityPolicyRule("policyRuleTwo",
            region="us-west2",
            security_policy=default.name,
            description="new rule two",
            priority=101,
            match=gcp.compute.RegionSecurityPolicyRuleMatchArgs(
                versioned_expr="SRC_IPS_V1",
                config=gcp.compute.RegionSecurityPolicyRuleMatchConfigArgs(
                    src_ip_ranges=[
                        "192.168.0.0/16",
                        "10.0.0.0/8",
                    ],
                ),
            ),
            action="allow",
            preview=True,
            opts=pulumi.ResourceOptions(provider=google_beta))
        ```
        ### Region Security Policy Rule With Network Match

        ```python
        import pulumi
        import pulumi_gcp as gcp

        # First activate advanced network DDoS protection for the desired region
        policyddosprotection = gcp.compute.RegionSecurityPolicy("policyddosprotection",
            region="us-west2",
            description="policy for activating network DDoS protection for the desired region",
            type="CLOUD_ARMOR_NETWORK",
            ddos_protection_config=gcp.compute.RegionSecurityPolicyDdosProtectionConfigArgs(
                ddos_protection="ADVANCED_PREVIEW",
            ),
            opts=pulumi.ResourceOptions(provider=google_beta))
        edge_sec_service = gcp.compute.NetworkEdgeSecurityService("edgeSecService",
            region="us-west2",
            description="linking policy to edge security service",
            security_policy=policyddosprotection.self_link,
            opts=pulumi.ResourceOptions(provider=google_beta))
        # Add the desired policy and custom rule.
        policynetworkmatch = gcp.compute.RegionSecurityPolicy("policynetworkmatch",
            region="us-west2",
            description="region security policy for network match",
            type="CLOUD_ARMOR_NETWORK",
            user_defined_fields=[gcp.compute.RegionSecurityPolicyUserDefinedFieldArgs(
                name="SIG1_AT_0",
                base="TCP",
                offset=8,
                size=2,
                mask="0x8F00",
            )],
            opts=pulumi.ResourceOptions(provider=google_beta,
                depends_on=[edge_sec_service]))
        policy_rule_network_match = gcp.compute.RegionSecurityPolicyRule("policyRuleNetworkMatch",
            region="us-west2",
            security_policy=policynetworkmatch.name,
            description="custom rule for network match",
            priority=100,
            network_match=gcp.compute.RegionSecurityPolicyRuleNetworkMatchArgs(
                src_ip_ranges=["10.10.0.0/16"],
                user_defined_fields=[gcp.compute.RegionSecurityPolicyRuleNetworkMatchUserDefinedFieldArgs(
                    name="SIG1_AT_0",
                    values=["0x8F00"],
                )],
            ),
            action="allow",
            preview=True,
            opts=pulumi.ResourceOptions(provider=google_beta))
        ```

        ## Import

        RegionSecurityPolicyRule can be imported using any of these accepted formats

        ```sh
         $ pulumi import gcp:compute/regionSecurityPolicyRule:RegionSecurityPolicyRule default projects/{{project}}/regions/{{region}}/securityPolicies/{{security_policy}}/priority/{{priority}}
        ```

        ```sh
         $ pulumi import gcp:compute/regionSecurityPolicyRule:RegionSecurityPolicyRule default {{project}}/{{region}}/{{security_policy}}/{{priority}}
        ```

        ```sh
         $ pulumi import gcp:compute/regionSecurityPolicyRule:RegionSecurityPolicyRule default {{region}}/{{security_policy}}/{{priority}}
        ```

        ```sh
         $ pulumi import gcp:compute/regionSecurityPolicyRule:RegionSecurityPolicyRule default {{security_policy}}/{{priority}}
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] action: The Action to perform when the rule is matched. The following are the valid actions:
               * allow: allow access to target.
               * deny(STATUS): deny access to target, returns the HTTP response code specified. Valid values for STATUS are 403, 404, and 502.
               * rate_based_ban: limit client traffic to the configured threshold and ban the client if the traffic exceeds the threshold. Configure parameters for this action in RateLimitOptions. Requires rateLimitOptions to be set.
               * redirect: redirect to a different target. This can either be an internal reCAPTCHA redirect, or an external URL-based redirect via a 302 response. Parameters for this action can be configured via redirectOptions. This action is only supported in Global Security Policies of type CLOUD_ARMOR.
               * throttle: limit client traffic to the configured threshold. Configure parameters for this action in rateLimitOptions. Requires rateLimitOptions to be set for this.
        :param pulumi.Input[str] description: An optional description of this resource. Provide this property when you create the resource.
        :param pulumi.Input[pulumi.InputType['RegionSecurityPolicyRuleMatchArgs']] match: A match condition that incoming traffic is evaluated against.
               If it evaluates to true, the corresponding 'action' is enforced.
               Structure is documented below.
        :param pulumi.Input[pulumi.InputType['RegionSecurityPolicyRuleNetworkMatchArgs']] network_match: A match condition that incoming packets are evaluated against for CLOUD_ARMOR_NETWORK security policies. If it matches, the corresponding 'action' is enforced.
               The match criteria for a rule consists of built-in match fields (like 'srcIpRanges') and potentially multiple user-defined match fields ('userDefinedFields').
               Field values may be extracted directly from the packet or derived from it (e.g. 'srcRegionCodes'). Some fields may not be present in every packet (e.g. 'srcPorts'). A user-defined field is only present if the base header is found in the packet and the entire field is in bounds.
               Each match field may specify which values can match it, listing one or more ranges, prefixes, or exact values that are considered a match for the field. A field value must be present in order to match a specified match field. If no match values are specified for a match field, then any field value is considered to match it, and it's not required to be present. For strings specifying '*' is also equivalent to match all.
               For a packet to match a rule, all specified match fields must match the corresponding field values derived from the packet.
               Example:
               networkMatch: srcIpRanges: - "192.0.2.0/24" - "198.51.100.0/24" userDefinedFields: - name: "ipv4_fragment_offset" values: - "1-0x1fff"
               The above match condition matches packets with a source IP in 192.0.2.0/24 or 198.51.100.0/24 and a user-defined field named "ipv4_fragment_offset" with a value between 1 and 0x1fff inclusive
               Structure is documented below.
        :param pulumi.Input[bool] preview: If set to true, the specified action is not enforced.
        :param pulumi.Input[int] priority: An integer indicating the priority of a rule in the list.
               The priority must be a positive value between 0 and 2147483647.
               Rules are evaluated from highest to lowest priority where 0 is the highest priority and 2147483647 is the lowest priority.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[str] region: The Region in which the created Region Security Policy rule should reside.
        :param pulumi.Input[str] security_policy: The name of the security policy this rule belongs to.
               
               
               - - -
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: RegionSecurityPolicyRuleArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        ## Example Usage
        ### Region Security Policy Rule Basic

        ```python
        import pulumi
        import pulumi_gcp as gcp

        default = gcp.compute.RegionSecurityPolicy("default",
            region="us-west2",
            description="basic region security policy",
            type="CLOUD_ARMOR",
            opts=pulumi.ResourceOptions(provider=google_beta))
        policy_rule = gcp.compute.RegionSecurityPolicyRule("policyRule",
            region="us-west2",
            security_policy=default.name,
            description="new rule",
            priority=100,
            match=gcp.compute.RegionSecurityPolicyRuleMatchArgs(
                versioned_expr="SRC_IPS_V1",
                config=gcp.compute.RegionSecurityPolicyRuleMatchConfigArgs(
                    src_ip_ranges=["10.10.0.0/16"],
                ),
            ),
            action="allow",
            preview=True,
            opts=pulumi.ResourceOptions(provider=google_beta))
        ```
        ### Region Security Policy Rule Multiple Rules

        ```python
        import pulumi
        import pulumi_gcp as gcp

        default = gcp.compute.RegionSecurityPolicy("default",
            region="us-west2",
            description="basic region security policy",
            type="CLOUD_ARMOR",
            opts=pulumi.ResourceOptions(provider=google_beta))
        policy_rule_one = gcp.compute.RegionSecurityPolicyRule("policyRuleOne",
            region="us-west2",
            security_policy=default.name,
            description="new rule one",
            priority=100,
            match=gcp.compute.RegionSecurityPolicyRuleMatchArgs(
                versioned_expr="SRC_IPS_V1",
                config=gcp.compute.RegionSecurityPolicyRuleMatchConfigArgs(
                    src_ip_ranges=["10.10.0.0/16"],
                ),
            ),
            action="allow",
            preview=True,
            opts=pulumi.ResourceOptions(provider=google_beta))
        policy_rule_two = gcp.compute.RegionSecurityPolicyRule("policyRuleTwo",
            region="us-west2",
            security_policy=default.name,
            description="new rule two",
            priority=101,
            match=gcp.compute.RegionSecurityPolicyRuleMatchArgs(
                versioned_expr="SRC_IPS_V1",
                config=gcp.compute.RegionSecurityPolicyRuleMatchConfigArgs(
                    src_ip_ranges=[
                        "192.168.0.0/16",
                        "10.0.0.0/8",
                    ],
                ),
            ),
            action="allow",
            preview=True,
            opts=pulumi.ResourceOptions(provider=google_beta))
        ```
        ### Region Security Policy Rule With Network Match

        ```python
        import pulumi
        import pulumi_gcp as gcp

        # First activate advanced network DDoS protection for the desired region
        policyddosprotection = gcp.compute.RegionSecurityPolicy("policyddosprotection",
            region="us-west2",
            description="policy for activating network DDoS protection for the desired region",
            type="CLOUD_ARMOR_NETWORK",
            ddos_protection_config=gcp.compute.RegionSecurityPolicyDdosProtectionConfigArgs(
                ddos_protection="ADVANCED_PREVIEW",
            ),
            opts=pulumi.ResourceOptions(provider=google_beta))
        edge_sec_service = gcp.compute.NetworkEdgeSecurityService("edgeSecService",
            region="us-west2",
            description="linking policy to edge security service",
            security_policy=policyddosprotection.self_link,
            opts=pulumi.ResourceOptions(provider=google_beta))
        # Add the desired policy and custom rule.
        policynetworkmatch = gcp.compute.RegionSecurityPolicy("policynetworkmatch",
            region="us-west2",
            description="region security policy for network match",
            type="CLOUD_ARMOR_NETWORK",
            user_defined_fields=[gcp.compute.RegionSecurityPolicyUserDefinedFieldArgs(
                name="SIG1_AT_0",
                base="TCP",
                offset=8,
                size=2,
                mask="0x8F00",
            )],
            opts=pulumi.ResourceOptions(provider=google_beta,
                depends_on=[edge_sec_service]))
        policy_rule_network_match = gcp.compute.RegionSecurityPolicyRule("policyRuleNetworkMatch",
            region="us-west2",
            security_policy=policynetworkmatch.name,
            description="custom rule for network match",
            priority=100,
            network_match=gcp.compute.RegionSecurityPolicyRuleNetworkMatchArgs(
                src_ip_ranges=["10.10.0.0/16"],
                user_defined_fields=[gcp.compute.RegionSecurityPolicyRuleNetworkMatchUserDefinedFieldArgs(
                    name="SIG1_AT_0",
                    values=["0x8F00"],
                )],
            ),
            action="allow",
            preview=True,
            opts=pulumi.ResourceOptions(provider=google_beta))
        ```

        ## Import

        RegionSecurityPolicyRule can be imported using any of these accepted formats

        ```sh
         $ pulumi import gcp:compute/regionSecurityPolicyRule:RegionSecurityPolicyRule default projects/{{project}}/regions/{{region}}/securityPolicies/{{security_policy}}/priority/{{priority}}
        ```

        ```sh
         $ pulumi import gcp:compute/regionSecurityPolicyRule:RegionSecurityPolicyRule default {{project}}/{{region}}/{{security_policy}}/{{priority}}
        ```

        ```sh
         $ pulumi import gcp:compute/regionSecurityPolicyRule:RegionSecurityPolicyRule default {{region}}/{{security_policy}}/{{priority}}
        ```

        ```sh
         $ pulumi import gcp:compute/regionSecurityPolicyRule:RegionSecurityPolicyRule default {{security_policy}}/{{priority}}
        ```

        :param str resource_name: The name of the resource.
        :param RegionSecurityPolicyRuleArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(RegionSecurityPolicyRuleArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 action: Optional[pulumi.Input[str]] = None,
                 description: Optional[pulumi.Input[str]] = None,
                 match: Optional[pulumi.Input[pulumi.InputType['RegionSecurityPolicyRuleMatchArgs']]] = None,
                 network_match: Optional[pulumi.Input[pulumi.InputType['RegionSecurityPolicyRuleNetworkMatchArgs']]] = None,
                 preview: Optional[pulumi.Input[bool]] = None,
                 priority: Optional[pulumi.Input[int]] = None,
                 project: Optional[pulumi.Input[str]] = None,
                 region: Optional[pulumi.Input[str]] = None,
                 security_policy: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = RegionSecurityPolicyRuleArgs.__new__(RegionSecurityPolicyRuleArgs)

            if action is None and not opts.urn:
                raise TypeError("Missing required property 'action'")
            __props__.__dict__["action"] = action
            __props__.__dict__["description"] = description
            __props__.__dict__["match"] = match
            __props__.__dict__["network_match"] = network_match
            __props__.__dict__["preview"] = preview
            if priority is None and not opts.urn:
                raise TypeError("Missing required property 'priority'")
            __props__.__dict__["priority"] = priority
            __props__.__dict__["project"] = project
            if region is None and not opts.urn:
                raise TypeError("Missing required property 'region'")
            __props__.__dict__["region"] = region
            if security_policy is None and not opts.urn:
                raise TypeError("Missing required property 'security_policy'")
            __props__.__dict__["security_policy"] = security_policy
        super(RegionSecurityPolicyRule, __self__).__init__(
            'gcp:compute/regionSecurityPolicyRule:RegionSecurityPolicyRule',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            action: Optional[pulumi.Input[str]] = None,
            description: Optional[pulumi.Input[str]] = None,
            match: Optional[pulumi.Input[pulumi.InputType['RegionSecurityPolicyRuleMatchArgs']]] = None,
            network_match: Optional[pulumi.Input[pulumi.InputType['RegionSecurityPolicyRuleNetworkMatchArgs']]] = None,
            preview: Optional[pulumi.Input[bool]] = None,
            priority: Optional[pulumi.Input[int]] = None,
            project: Optional[pulumi.Input[str]] = None,
            region: Optional[pulumi.Input[str]] = None,
            security_policy: Optional[pulumi.Input[str]] = None) -> 'RegionSecurityPolicyRule':
        """
        Get an existing RegionSecurityPolicyRule resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] action: The Action to perform when the rule is matched. The following are the valid actions:
               * allow: allow access to target.
               * deny(STATUS): deny access to target, returns the HTTP response code specified. Valid values for STATUS are 403, 404, and 502.
               * rate_based_ban: limit client traffic to the configured threshold and ban the client if the traffic exceeds the threshold. Configure parameters for this action in RateLimitOptions. Requires rateLimitOptions to be set.
               * redirect: redirect to a different target. This can either be an internal reCAPTCHA redirect, or an external URL-based redirect via a 302 response. Parameters for this action can be configured via redirectOptions. This action is only supported in Global Security Policies of type CLOUD_ARMOR.
               * throttle: limit client traffic to the configured threshold. Configure parameters for this action in rateLimitOptions. Requires rateLimitOptions to be set for this.
        :param pulumi.Input[str] description: An optional description of this resource. Provide this property when you create the resource.
        :param pulumi.Input[pulumi.InputType['RegionSecurityPolicyRuleMatchArgs']] match: A match condition that incoming traffic is evaluated against.
               If it evaluates to true, the corresponding 'action' is enforced.
               Structure is documented below.
        :param pulumi.Input[pulumi.InputType['RegionSecurityPolicyRuleNetworkMatchArgs']] network_match: A match condition that incoming packets are evaluated against for CLOUD_ARMOR_NETWORK security policies. If it matches, the corresponding 'action' is enforced.
               The match criteria for a rule consists of built-in match fields (like 'srcIpRanges') and potentially multiple user-defined match fields ('userDefinedFields').
               Field values may be extracted directly from the packet or derived from it (e.g. 'srcRegionCodes'). Some fields may not be present in every packet (e.g. 'srcPorts'). A user-defined field is only present if the base header is found in the packet and the entire field is in bounds.
               Each match field may specify which values can match it, listing one or more ranges, prefixes, or exact values that are considered a match for the field. A field value must be present in order to match a specified match field. If no match values are specified for a match field, then any field value is considered to match it, and it's not required to be present. For strings specifying '*' is also equivalent to match all.
               For a packet to match a rule, all specified match fields must match the corresponding field values derived from the packet.
               Example:
               networkMatch: srcIpRanges: - "192.0.2.0/24" - "198.51.100.0/24" userDefinedFields: - name: "ipv4_fragment_offset" values: - "1-0x1fff"
               The above match condition matches packets with a source IP in 192.0.2.0/24 or 198.51.100.0/24 and a user-defined field named "ipv4_fragment_offset" with a value between 1 and 0x1fff inclusive
               Structure is documented below.
        :param pulumi.Input[bool] preview: If set to true, the specified action is not enforced.
        :param pulumi.Input[int] priority: An integer indicating the priority of a rule in the list.
               The priority must be a positive value between 0 and 2147483647.
               Rules are evaluated from highest to lowest priority where 0 is the highest priority and 2147483647 is the lowest priority.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[str] region: The Region in which the created Region Security Policy rule should reside.
        :param pulumi.Input[str] security_policy: The name of the security policy this rule belongs to.
               
               
               - - -
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _RegionSecurityPolicyRuleState.__new__(_RegionSecurityPolicyRuleState)

        __props__.__dict__["action"] = action
        __props__.__dict__["description"] = description
        __props__.__dict__["match"] = match
        __props__.__dict__["network_match"] = network_match
        __props__.__dict__["preview"] = preview
        __props__.__dict__["priority"] = priority
        __props__.__dict__["project"] = project
        __props__.__dict__["region"] = region
        __props__.__dict__["security_policy"] = security_policy
        return RegionSecurityPolicyRule(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter
    def action(self) -> pulumi.Output[str]:
        """
        The Action to perform when the rule is matched. The following are the valid actions:
        * allow: allow access to target.
        * deny(STATUS): deny access to target, returns the HTTP response code specified. Valid values for STATUS are 403, 404, and 502.
        * rate_based_ban: limit client traffic to the configured threshold and ban the client if the traffic exceeds the threshold. Configure parameters for this action in RateLimitOptions. Requires rateLimitOptions to be set.
        * redirect: redirect to a different target. This can either be an internal reCAPTCHA redirect, or an external URL-based redirect via a 302 response. Parameters for this action can be configured via redirectOptions. This action is only supported in Global Security Policies of type CLOUD_ARMOR.
        * throttle: limit client traffic to the configured threshold. Configure parameters for this action in rateLimitOptions. Requires rateLimitOptions to be set for this.
        """
        return pulumi.get(self, "action")

    @property
    @pulumi.getter
    def description(self) -> pulumi.Output[Optional[str]]:
        """
        An optional description of this resource. Provide this property when you create the resource.
        """
        return pulumi.get(self, "description")

    @property
    @pulumi.getter
    def match(self) -> pulumi.Output[Optional['outputs.RegionSecurityPolicyRuleMatch']]:
        """
        A match condition that incoming traffic is evaluated against.
        If it evaluates to true, the corresponding 'action' is enforced.
        Structure is documented below.
        """
        return pulumi.get(self, "match")

    @property
    @pulumi.getter(name="networkMatch")
    def network_match(self) -> pulumi.Output[Optional['outputs.RegionSecurityPolicyRuleNetworkMatch']]:
        """
        A match condition that incoming packets are evaluated against for CLOUD_ARMOR_NETWORK security policies. If it matches, the corresponding 'action' is enforced.
        The match criteria for a rule consists of built-in match fields (like 'srcIpRanges') and potentially multiple user-defined match fields ('userDefinedFields').
        Field values may be extracted directly from the packet or derived from it (e.g. 'srcRegionCodes'). Some fields may not be present in every packet (e.g. 'srcPorts'). A user-defined field is only present if the base header is found in the packet and the entire field is in bounds.
        Each match field may specify which values can match it, listing one or more ranges, prefixes, or exact values that are considered a match for the field. A field value must be present in order to match a specified match field. If no match values are specified for a match field, then any field value is considered to match it, and it's not required to be present. For strings specifying '*' is also equivalent to match all.
        For a packet to match a rule, all specified match fields must match the corresponding field values derived from the packet.
        Example:
        networkMatch: srcIpRanges: - "192.0.2.0/24" - "198.51.100.0/24" userDefinedFields: - name: "ipv4_fragment_offset" values: - "1-0x1fff"
        The above match condition matches packets with a source IP in 192.0.2.0/24 or 198.51.100.0/24 and a user-defined field named "ipv4_fragment_offset" with a value between 1 and 0x1fff inclusive
        Structure is documented below.
        """
        return pulumi.get(self, "network_match")

    @property
    @pulumi.getter
    def preview(self) -> pulumi.Output[Optional[bool]]:
        """
        If set to true, the specified action is not enforced.
        """
        return pulumi.get(self, "preview")

    @property
    @pulumi.getter
    def priority(self) -> pulumi.Output[int]:
        """
        An integer indicating the priority of a rule in the list.
        The priority must be a positive value between 0 and 2147483647.
        Rules are evaluated from highest to lowest priority where 0 is the highest priority and 2147483647 is the lowest priority.
        """
        return pulumi.get(self, "priority")

    @property
    @pulumi.getter
    def project(self) -> pulumi.Output[str]:
        """
        The ID of the project in which the resource belongs.
        If it is not provided, the provider project is used.
        """
        return pulumi.get(self, "project")

    @property
    @pulumi.getter
    def region(self) -> pulumi.Output[str]:
        """
        The Region in which the created Region Security Policy rule should reside.
        """
        return pulumi.get(self, "region")

    @property
    @pulumi.getter(name="securityPolicy")
    def security_policy(self) -> pulumi.Output[str]:
        """
        The name of the security policy this rule belongs to.


        - - -
        """
        return pulumi.get(self, "security_policy")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from . import outputs
from ._inputs import *

__all__ = ['KeystoresAliasesSelfSignedCertArgs', 'KeystoresAliasesSelfSignedCert']

@pulumi.input_type
class KeystoresAliasesSelfSignedCertArgs:
    def __init__(__self__, *,
                 alias: pulumi.Input[str],
                 environment: pulumi.Input[str],
                 keystore: pulumi.Input[str],
                 org_id: pulumi.Input[str],
                 sig_alg: pulumi.Input[str],
                 subject: pulumi.Input['KeystoresAliasesSelfSignedCertSubjectArgs'],
                 cert_validity_in_days: Optional[pulumi.Input[int]] = None,
                 key_size: Optional[pulumi.Input[str]] = None,
                 subject_alternative_dns_names: Optional[pulumi.Input['KeystoresAliasesSelfSignedCertSubjectAlternativeDnsNamesArgs']] = None):
        """
        The set of arguments for constructing a KeystoresAliasesSelfSignedCert resource.
        :param pulumi.Input[str] alias: Alias for the key/certificate pair. Values must match the regular expression [\\w\\s-.]{1,255}.
               This must be provided for all formats except selfsignedcert; self-signed certs may specify the alias in either
               this parameter or the JSON body.
        :param pulumi.Input[str] environment: The Apigee environment name
        :param pulumi.Input[str] keystore: The Apigee keystore name associated in an Apigee environment
        :param pulumi.Input[str] org_id: The Apigee Organization name associated with the Apigee environment
        :param pulumi.Input[str] sig_alg: Signature algorithm to generate private key. Valid values are SHA512withRSA, SHA384withRSA, and SHA256withRSA
        :param pulumi.Input['KeystoresAliasesSelfSignedCertSubjectArgs'] subject: Subject details.
               Structure is documented below.
        :param pulumi.Input[int] cert_validity_in_days: Validity duration of certificate, in days. Accepts positive non-zero value. Defaults to 365.
        :param pulumi.Input[str] key_size: Key size. Default and maximum value is 2048 bits.
        :param pulumi.Input['KeystoresAliasesSelfSignedCertSubjectAlternativeDnsNamesArgs'] subject_alternative_dns_names: List of alternative host names. Maximum length is 255 characters for each value.
               Structure is documented below.
        """
        pulumi.set(__self__, "alias", alias)
        pulumi.set(__self__, "environment", environment)
        pulumi.set(__self__, "keystore", keystore)
        pulumi.set(__self__, "org_id", org_id)
        pulumi.set(__self__, "sig_alg", sig_alg)
        pulumi.set(__self__, "subject", subject)
        if cert_validity_in_days is not None:
            pulumi.set(__self__, "cert_validity_in_days", cert_validity_in_days)
        if key_size is not None:
            pulumi.set(__self__, "key_size", key_size)
        if subject_alternative_dns_names is not None:
            pulumi.set(__self__, "subject_alternative_dns_names", subject_alternative_dns_names)

    @property
    @pulumi.getter
    def alias(self) -> pulumi.Input[str]:
        """
        Alias for the key/certificate pair. Values must match the regular expression [\\w\\s-.]{1,255}.
        This must be provided for all formats except selfsignedcert; self-signed certs may specify the alias in either
        this parameter or the JSON body.
        """
        return pulumi.get(self, "alias")

    @alias.setter
    def alias(self, value: pulumi.Input[str]):
        pulumi.set(self, "alias", value)

    @property
    @pulumi.getter
    def environment(self) -> pulumi.Input[str]:
        """
        The Apigee environment name
        """
        return pulumi.get(self, "environment")

    @environment.setter
    def environment(self, value: pulumi.Input[str]):
        pulumi.set(self, "environment", value)

    @property
    @pulumi.getter
    def keystore(self) -> pulumi.Input[str]:
        """
        The Apigee keystore name associated in an Apigee environment
        """
        return pulumi.get(self, "keystore")

    @keystore.setter
    def keystore(self, value: pulumi.Input[str]):
        pulumi.set(self, "keystore", value)

    @property
    @pulumi.getter(name="orgId")
    def org_id(self) -> pulumi.Input[str]:
        """
        The Apigee Organization name associated with the Apigee environment
        """
        return pulumi.get(self, "org_id")

    @org_id.setter
    def org_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "org_id", value)

    @property
    @pulumi.getter(name="sigAlg")
    def sig_alg(self) -> pulumi.Input[str]:
        """
        Signature algorithm to generate private key. Valid values are SHA512withRSA, SHA384withRSA, and SHA256withRSA
        """
        return pulumi.get(self, "sig_alg")

    @sig_alg.setter
    def sig_alg(self, value: pulumi.Input[str]):
        pulumi.set(self, "sig_alg", value)

    @property
    @pulumi.getter
    def subject(self) -> pulumi.Input['KeystoresAliasesSelfSignedCertSubjectArgs']:
        """
        Subject details.
        Structure is documented below.
        """
        return pulumi.get(self, "subject")

    @subject.setter
    def subject(self, value: pulumi.Input['KeystoresAliasesSelfSignedCertSubjectArgs']):
        pulumi.set(self, "subject", value)

    @property
    @pulumi.getter(name="certValidityInDays")
    def cert_validity_in_days(self) -> Optional[pulumi.Input[int]]:
        """
        Validity duration of certificate, in days. Accepts positive non-zero value. Defaults to 365.
        """
        return pulumi.get(self, "cert_validity_in_days")

    @cert_validity_in_days.setter
    def cert_validity_in_days(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "cert_validity_in_days", value)

    @property
    @pulumi.getter(name="keySize")
    def key_size(self) -> Optional[pulumi.Input[str]]:
        """
        Key size. Default and maximum value is 2048 bits.
        """
        return pulumi.get(self, "key_size")

    @key_size.setter
    def key_size(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "key_size", value)

    @property
    @pulumi.getter(name="subjectAlternativeDnsNames")
    def subject_alternative_dns_names(self) -> Optional[pulumi.Input['KeystoresAliasesSelfSignedCertSubjectAlternativeDnsNamesArgs']]:
        """
        List of alternative host names. Maximum length is 255 characters for each value.
        Structure is documented below.
        """
        return pulumi.get(self, "subject_alternative_dns_names")

    @subject_alternative_dns_names.setter
    def subject_alternative_dns_names(self, value: Optional[pulumi.Input['KeystoresAliasesSelfSignedCertSubjectAlternativeDnsNamesArgs']]):
        pulumi.set(self, "subject_alternative_dns_names", value)


@pulumi.input_type
class _KeystoresAliasesSelfSignedCertState:
    def __init__(__self__, *,
                 alias: Optional[pulumi.Input[str]] = None,
                 cert_validity_in_days: Optional[pulumi.Input[int]] = None,
                 certs_infos: Optional[pulumi.Input[Sequence[pulumi.Input['KeystoresAliasesSelfSignedCertCertsInfoArgs']]]] = None,
                 environment: Optional[pulumi.Input[str]] = None,
                 key_size: Optional[pulumi.Input[str]] = None,
                 keystore: Optional[pulumi.Input[str]] = None,
                 org_id: Optional[pulumi.Input[str]] = None,
                 sig_alg: Optional[pulumi.Input[str]] = None,
                 subject: Optional[pulumi.Input['KeystoresAliasesSelfSignedCertSubjectArgs']] = None,
                 subject_alternative_dns_names: Optional[pulumi.Input['KeystoresAliasesSelfSignedCertSubjectAlternativeDnsNamesArgs']] = None,
                 type: Optional[pulumi.Input[str]] = None):
        """
        Input properties used for looking up and filtering KeystoresAliasesSelfSignedCert resources.
        :param pulumi.Input[str] alias: Alias for the key/certificate pair. Values must match the regular expression [\\w\\s-.]{1,255}.
               This must be provided for all formats except selfsignedcert; self-signed certs may specify the alias in either
               this parameter or the JSON body.
        :param pulumi.Input[int] cert_validity_in_days: Validity duration of certificate, in days. Accepts positive non-zero value. Defaults to 365.
        :param pulumi.Input[Sequence[pulumi.Input['KeystoresAliasesSelfSignedCertCertsInfoArgs']]] certs_infos: Chain of certificates under this alias.
               Structure is documented below.
        :param pulumi.Input[str] environment: The Apigee environment name
        :param pulumi.Input[str] key_size: Key size. Default and maximum value is 2048 bits.
        :param pulumi.Input[str] keystore: The Apigee keystore name associated in an Apigee environment
        :param pulumi.Input[str] org_id: The Apigee Organization name associated with the Apigee environment
        :param pulumi.Input[str] sig_alg: Signature algorithm to generate private key. Valid values are SHA512withRSA, SHA384withRSA, and SHA256withRSA
        :param pulumi.Input['KeystoresAliasesSelfSignedCertSubjectArgs'] subject: Subject details.
               Structure is documented below.
        :param pulumi.Input['KeystoresAliasesSelfSignedCertSubjectAlternativeDnsNamesArgs'] subject_alternative_dns_names: List of alternative host names. Maximum length is 255 characters for each value.
               Structure is documented below.
        :param pulumi.Input[str] type: Optional.Type of Alias
        """
        if alias is not None:
            pulumi.set(__self__, "alias", alias)
        if cert_validity_in_days is not None:
            pulumi.set(__self__, "cert_validity_in_days", cert_validity_in_days)
        if certs_infos is not None:
            pulumi.set(__self__, "certs_infos", certs_infos)
        if environment is not None:
            pulumi.set(__self__, "environment", environment)
        if key_size is not None:
            pulumi.set(__self__, "key_size", key_size)
        if keystore is not None:
            pulumi.set(__self__, "keystore", keystore)
        if org_id is not None:
            pulumi.set(__self__, "org_id", org_id)
        if sig_alg is not None:
            pulumi.set(__self__, "sig_alg", sig_alg)
        if subject is not None:
            pulumi.set(__self__, "subject", subject)
        if subject_alternative_dns_names is not None:
            pulumi.set(__self__, "subject_alternative_dns_names", subject_alternative_dns_names)
        if type is not None:
            pulumi.set(__self__, "type", type)

    @property
    @pulumi.getter
    def alias(self) -> Optional[pulumi.Input[str]]:
        """
        Alias for the key/certificate pair. Values must match the regular expression [\\w\\s-.]{1,255}.
        This must be provided for all formats except selfsignedcert; self-signed certs may specify the alias in either
        this parameter or the JSON body.
        """
        return pulumi.get(self, "alias")

    @alias.setter
    def alias(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "alias", value)

    @property
    @pulumi.getter(name="certValidityInDays")
    def cert_validity_in_days(self) -> Optional[pulumi.Input[int]]:
        """
        Validity duration of certificate, in days. Accepts positive non-zero value. Defaults to 365.
        """
        return pulumi.get(self, "cert_validity_in_days")

    @cert_validity_in_days.setter
    def cert_validity_in_days(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "cert_validity_in_days", value)

    @property
    @pulumi.getter(name="certsInfos")
    def certs_infos(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['KeystoresAliasesSelfSignedCertCertsInfoArgs']]]]:
        """
        Chain of certificates under this alias.
        Structure is documented below.
        """
        return pulumi.get(self, "certs_infos")

    @certs_infos.setter
    def certs_infos(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['KeystoresAliasesSelfSignedCertCertsInfoArgs']]]]):
        pulumi.set(self, "certs_infos", value)

    @property
    @pulumi.getter
    def environment(self) -> Optional[pulumi.Input[str]]:
        """
        The Apigee environment name
        """
        return pulumi.get(self, "environment")

    @environment.setter
    def environment(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "environment", value)

    @property
    @pulumi.getter(name="keySize")
    def key_size(self) -> Optional[pulumi.Input[str]]:
        """
        Key size. Default and maximum value is 2048 bits.
        """
        return pulumi.get(self, "key_size")

    @key_size.setter
    def key_size(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "key_size", value)

    @property
    @pulumi.getter
    def keystore(self) -> Optional[pulumi.Input[str]]:
        """
        The Apigee keystore name associated in an Apigee environment
        """
        return pulumi.get(self, "keystore")

    @keystore.setter
    def keystore(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "keystore", value)

    @property
    @pulumi.getter(name="orgId")
    def org_id(self) -> Optional[pulumi.Input[str]]:
        """
        The Apigee Organization name associated with the Apigee environment
        """
        return pulumi.get(self, "org_id")

    @org_id.setter
    def org_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "org_id", value)

    @property
    @pulumi.getter(name="sigAlg")
    def sig_alg(self) -> Optional[pulumi.Input[str]]:
        """
        Signature algorithm to generate private key. Valid values are SHA512withRSA, SHA384withRSA, and SHA256withRSA
        """
        return pulumi.get(self, "sig_alg")

    @sig_alg.setter
    def sig_alg(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "sig_alg", value)

    @property
    @pulumi.getter
    def subject(self) -> Optional[pulumi.Input['KeystoresAliasesSelfSignedCertSubjectArgs']]:
        """
        Subject details.
        Structure is documented below.
        """
        return pulumi.get(self, "subject")

    @subject.setter
    def subject(self, value: Optional[pulumi.Input['KeystoresAliasesSelfSignedCertSubjectArgs']]):
        pulumi.set(self, "subject", value)

    @property
    @pulumi.getter(name="subjectAlternativeDnsNames")
    def subject_alternative_dns_names(self) -> Optional[pulumi.Input['KeystoresAliasesSelfSignedCertSubjectAlternativeDnsNamesArgs']]:
        """
        List of alternative host names. Maximum length is 255 characters for each value.
        Structure is documented below.
        """
        return pulumi.get(self, "subject_alternative_dns_names")

    @subject_alternative_dns_names.setter
    def subject_alternative_dns_names(self, value: Optional[pulumi.Input['KeystoresAliasesSelfSignedCertSubjectAlternativeDnsNamesArgs']]):
        pulumi.set(self, "subject_alternative_dns_names", value)

    @property
    @pulumi.getter
    def type(self) -> Optional[pulumi.Input[str]]:
        """
        Optional.Type of Alias
        """
        return pulumi.get(self, "type")

    @type.setter
    def type(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "type", value)


class KeystoresAliasesSelfSignedCert(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 alias: Optional[pulumi.Input[str]] = None,
                 cert_validity_in_days: Optional[pulumi.Input[int]] = None,
                 environment: Optional[pulumi.Input[str]] = None,
                 key_size: Optional[pulumi.Input[str]] = None,
                 keystore: Optional[pulumi.Input[str]] = None,
                 org_id: Optional[pulumi.Input[str]] = None,
                 sig_alg: Optional[pulumi.Input[str]] = None,
                 subject: Optional[pulumi.Input[pulumi.InputType['KeystoresAliasesSelfSignedCertSubjectArgs']]] = None,
                 subject_alternative_dns_names: Optional[pulumi.Input[pulumi.InputType['KeystoresAliasesSelfSignedCertSubjectAlternativeDnsNamesArgs']]] = None,
                 __props__=None):
        """
        An Environment Keystore Alias for Self Signed Certificate Format in Apigee

        To get more information about KeystoresAliasesSelfSignedCert, see:

        * [API documentation](https://cloud.google.com/apigee/docs/reference/apis/apigee/rest/v1/organizations.environments.keystores.aliases/create)
        * How-to Guides
            * [Creating an environment](https://cloud.google.com/apigee/docs/api-platform/get-started/create-environment)

        ## Example Usage
        ### Apigee Env Keystore Alias Self Signed Cert

        ```python
        import pulumi
        import pulumi_gcp as gcp

        project = gcp.organizations.Project("project",
            project_id="my-project",
            org_id="123456789",
            billing_account="000000-0000000-0000000-000000")
        apigee = gcp.projects.Service("apigee",
            project=project.project_id,
            service="apigee.googleapis.com")
        servicenetworking = gcp.projects.Service("servicenetworking",
            project=project.project_id,
            service="servicenetworking.googleapis.com",
            opts=pulumi.ResourceOptions(depends_on=[apigee]))
        compute = gcp.projects.Service("compute",
            project=project.project_id,
            service="compute.googleapis.com",
            opts=pulumi.ResourceOptions(depends_on=[servicenetworking]))
        apigee_network = gcp.compute.Network("apigeeNetwork", project=project.project_id,
        opts=pulumi.ResourceOptions(depends_on=[compute]))
        apigee_range = gcp.compute.GlobalAddress("apigeeRange",
            purpose="VPC_PEERING",
            address_type="INTERNAL",
            prefix_length=16,
            network=apigee_network.id,
            project=project.project_id)
        apigee_vpc_connection = gcp.servicenetworking.Connection("apigeeVpcConnection",
            network=apigee_network.id,
            service="servicenetworking.googleapis.com",
            reserved_peering_ranges=[apigee_range.name],
            opts=pulumi.ResourceOptions(depends_on=[servicenetworking]))
        apigee_org = gcp.apigee.Organization("apigeeOrg",
            analytics_region="us-central1",
            project_id=project.project_id,
            authorized_network=apigee_network.id,
            opts=pulumi.ResourceOptions(depends_on=[
                    apigee_vpc_connection,
                    apigee,
                ]))
        apigee_environment_keystore_ss_alias_environment = gcp.apigee.Environment("apigeeEnvironmentKeystoreSsAliasEnvironment",
            org_id=apigee_org.id,
            description="Apigee Environment",
            display_name="environment-1")
        apigee_environment_keystore_alias = gcp.apigee.EnvKeystore("apigeeEnvironmentKeystoreAlias", env_id=apigee_environment_keystore_ss_alias_environment.id)
        apigee_environment_keystore_ss_alias_keystores_aliases_self_signed_cert = gcp.apigee.KeystoresAliasesSelfSignedCert("apigeeEnvironmentKeystoreSsAliasKeystoresAliasesSelfSignedCert",
            environment=apigee_environment_keystore_ss_alias_environment.name,
            org_id=apigee_org.name,
            keystore=apigee_environment_keystore_alias.name,
            alias="alias",
            key_size="1024",
            sig_alg="SHA512withRSA",
            cert_validity_in_days=4,
            subject=gcp.apigee.KeystoresAliasesSelfSignedCertSubjectArgs(
                common_name="selfsigned_example",
                country_code="US",
                locality="TX",
                org="CCE",
                org_unit="PSO",
            ))
        ```

        ## Import

        KeystoresAliasesSelfSignedCert can be imported using any of these accepted formats

        ```sh
         $ pulumi import gcp:apigee/keystoresAliasesSelfSignedCert:KeystoresAliasesSelfSignedCert default organizations/{{org_id}}/environments/{{environment}}/keystores/{{keystore}}/aliases/{{alias}}
        ```

        ```sh
         $ pulumi import gcp:apigee/keystoresAliasesSelfSignedCert:KeystoresAliasesSelfSignedCert default {{org_id}}/{{environment}}/{{keystore}}/{{alias}}
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] alias: Alias for the key/certificate pair. Values must match the regular expression [\\w\\s-.]{1,255}.
               This must be provided for all formats except selfsignedcert; self-signed certs may specify the alias in either
               this parameter or the JSON body.
        :param pulumi.Input[int] cert_validity_in_days: Validity duration of certificate, in days. Accepts positive non-zero value. Defaults to 365.
        :param pulumi.Input[str] environment: The Apigee environment name
        :param pulumi.Input[str] key_size: Key size. Default and maximum value is 2048 bits.
        :param pulumi.Input[str] keystore: The Apigee keystore name associated in an Apigee environment
        :param pulumi.Input[str] org_id: The Apigee Organization name associated with the Apigee environment
        :param pulumi.Input[str] sig_alg: Signature algorithm to generate private key. Valid values are SHA512withRSA, SHA384withRSA, and SHA256withRSA
        :param pulumi.Input[pulumi.InputType['KeystoresAliasesSelfSignedCertSubjectArgs']] subject: Subject details.
               Structure is documented below.
        :param pulumi.Input[pulumi.InputType['KeystoresAliasesSelfSignedCertSubjectAlternativeDnsNamesArgs']] subject_alternative_dns_names: List of alternative host names. Maximum length is 255 characters for each value.
               Structure is documented below.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: KeystoresAliasesSelfSignedCertArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        An Environment Keystore Alias for Self Signed Certificate Format in Apigee

        To get more information about KeystoresAliasesSelfSignedCert, see:

        * [API documentation](https://cloud.google.com/apigee/docs/reference/apis/apigee/rest/v1/organizations.environments.keystores.aliases/create)
        * How-to Guides
            * [Creating an environment](https://cloud.google.com/apigee/docs/api-platform/get-started/create-environment)

        ## Example Usage
        ### Apigee Env Keystore Alias Self Signed Cert

        ```python
        import pulumi
        import pulumi_gcp as gcp

        project = gcp.organizations.Project("project",
            project_id="my-project",
            org_id="123456789",
            billing_account="000000-0000000-0000000-000000")
        apigee = gcp.projects.Service("apigee",
            project=project.project_id,
            service="apigee.googleapis.com")
        servicenetworking = gcp.projects.Service("servicenetworking",
            project=project.project_id,
            service="servicenetworking.googleapis.com",
            opts=pulumi.ResourceOptions(depends_on=[apigee]))
        compute = gcp.projects.Service("compute",
            project=project.project_id,
            service="compute.googleapis.com",
            opts=pulumi.ResourceOptions(depends_on=[servicenetworking]))
        apigee_network = gcp.compute.Network("apigeeNetwork", project=project.project_id,
        opts=pulumi.ResourceOptions(depends_on=[compute]))
        apigee_range = gcp.compute.GlobalAddress("apigeeRange",
            purpose="VPC_PEERING",
            address_type="INTERNAL",
            prefix_length=16,
            network=apigee_network.id,
            project=project.project_id)
        apigee_vpc_connection = gcp.servicenetworking.Connection("apigeeVpcConnection",
            network=apigee_network.id,
            service="servicenetworking.googleapis.com",
            reserved_peering_ranges=[apigee_range.name],
            opts=pulumi.ResourceOptions(depends_on=[servicenetworking]))
        apigee_org = gcp.apigee.Organization("apigeeOrg",
            analytics_region="us-central1",
            project_id=project.project_id,
            authorized_network=apigee_network.id,
            opts=pulumi.ResourceOptions(depends_on=[
                    apigee_vpc_connection,
                    apigee,
                ]))
        apigee_environment_keystore_ss_alias_environment = gcp.apigee.Environment("apigeeEnvironmentKeystoreSsAliasEnvironment",
            org_id=apigee_org.id,
            description="Apigee Environment",
            display_name="environment-1")
        apigee_environment_keystore_alias = gcp.apigee.EnvKeystore("apigeeEnvironmentKeystoreAlias", env_id=apigee_environment_keystore_ss_alias_environment.id)
        apigee_environment_keystore_ss_alias_keystores_aliases_self_signed_cert = gcp.apigee.KeystoresAliasesSelfSignedCert("apigeeEnvironmentKeystoreSsAliasKeystoresAliasesSelfSignedCert",
            environment=apigee_environment_keystore_ss_alias_environment.name,
            org_id=apigee_org.name,
            keystore=apigee_environment_keystore_alias.name,
            alias="alias",
            key_size="1024",
            sig_alg="SHA512withRSA",
            cert_validity_in_days=4,
            subject=gcp.apigee.KeystoresAliasesSelfSignedCertSubjectArgs(
                common_name="selfsigned_example",
                country_code="US",
                locality="TX",
                org="CCE",
                org_unit="PSO",
            ))
        ```

        ## Import

        KeystoresAliasesSelfSignedCert can be imported using any of these accepted formats

        ```sh
         $ pulumi import gcp:apigee/keystoresAliasesSelfSignedCert:KeystoresAliasesSelfSignedCert default organizations/{{org_id}}/environments/{{environment}}/keystores/{{keystore}}/aliases/{{alias}}
        ```

        ```sh
         $ pulumi import gcp:apigee/keystoresAliasesSelfSignedCert:KeystoresAliasesSelfSignedCert default {{org_id}}/{{environment}}/{{keystore}}/{{alias}}
        ```

        :param str resource_name: The name of the resource.
        :param KeystoresAliasesSelfSignedCertArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(KeystoresAliasesSelfSignedCertArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 alias: Optional[pulumi.Input[str]] = None,
                 cert_validity_in_days: Optional[pulumi.Input[int]] = None,
                 environment: Optional[pulumi.Input[str]] = None,
                 key_size: Optional[pulumi.Input[str]] = None,
                 keystore: Optional[pulumi.Input[str]] = None,
                 org_id: Optional[pulumi.Input[str]] = None,
                 sig_alg: Optional[pulumi.Input[str]] = None,
                 subject: Optional[pulumi.Input[pulumi.InputType['KeystoresAliasesSelfSignedCertSubjectArgs']]] = None,
                 subject_alternative_dns_names: Optional[pulumi.Input[pulumi.InputType['KeystoresAliasesSelfSignedCertSubjectAlternativeDnsNamesArgs']]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = KeystoresAliasesSelfSignedCertArgs.__new__(KeystoresAliasesSelfSignedCertArgs)

            if alias is None and not opts.urn:
                raise TypeError("Missing required property 'alias'")
            __props__.__dict__["alias"] = alias
            __props__.__dict__["cert_validity_in_days"] = cert_validity_in_days
            if environment is None and not opts.urn:
                raise TypeError("Missing required property 'environment'")
            __props__.__dict__["environment"] = environment
            __props__.__dict__["key_size"] = key_size
            if keystore is None and not opts.urn:
                raise TypeError("Missing required property 'keystore'")
            __props__.__dict__["keystore"] = keystore
            if org_id is None and not opts.urn:
                raise TypeError("Missing required property 'org_id'")
            __props__.__dict__["org_id"] = org_id
            if sig_alg is None and not opts.urn:
                raise TypeError("Missing required property 'sig_alg'")
            __props__.__dict__["sig_alg"] = sig_alg
            if subject is None and not opts.urn:
                raise TypeError("Missing required property 'subject'")
            __props__.__dict__["subject"] = subject
            __props__.__dict__["subject_alternative_dns_names"] = subject_alternative_dns_names
            __props__.__dict__["certs_infos"] = None
            __props__.__dict__["type"] = None
        super(KeystoresAliasesSelfSignedCert, __self__).__init__(
            'gcp:apigee/keystoresAliasesSelfSignedCert:KeystoresAliasesSelfSignedCert',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            alias: Optional[pulumi.Input[str]] = None,
            cert_validity_in_days: Optional[pulumi.Input[int]] = None,
            certs_infos: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['KeystoresAliasesSelfSignedCertCertsInfoArgs']]]]] = None,
            environment: Optional[pulumi.Input[str]] = None,
            key_size: Optional[pulumi.Input[str]] = None,
            keystore: Optional[pulumi.Input[str]] = None,
            org_id: Optional[pulumi.Input[str]] = None,
            sig_alg: Optional[pulumi.Input[str]] = None,
            subject: Optional[pulumi.Input[pulumi.InputType['KeystoresAliasesSelfSignedCertSubjectArgs']]] = None,
            subject_alternative_dns_names: Optional[pulumi.Input[pulumi.InputType['KeystoresAliasesSelfSignedCertSubjectAlternativeDnsNamesArgs']]] = None,
            type: Optional[pulumi.Input[str]] = None) -> 'KeystoresAliasesSelfSignedCert':
        """
        Get an existing KeystoresAliasesSelfSignedCert resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] alias: Alias for the key/certificate pair. Values must match the regular expression [\\w\\s-.]{1,255}.
               This must be provided for all formats except selfsignedcert; self-signed certs may specify the alias in either
               this parameter or the JSON body.
        :param pulumi.Input[int] cert_validity_in_days: Validity duration of certificate, in days. Accepts positive non-zero value. Defaults to 365.
        :param pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['KeystoresAliasesSelfSignedCertCertsInfoArgs']]]] certs_infos: Chain of certificates under this alias.
               Structure is documented below.
        :param pulumi.Input[str] environment: The Apigee environment name
        :param pulumi.Input[str] key_size: Key size. Default and maximum value is 2048 bits.
        :param pulumi.Input[str] keystore: The Apigee keystore name associated in an Apigee environment
        :param pulumi.Input[str] org_id: The Apigee Organization name associated with the Apigee environment
        :param pulumi.Input[str] sig_alg: Signature algorithm to generate private key. Valid values are SHA512withRSA, SHA384withRSA, and SHA256withRSA
        :param pulumi.Input[pulumi.InputType['KeystoresAliasesSelfSignedCertSubjectArgs']] subject: Subject details.
               Structure is documented below.
        :param pulumi.Input[pulumi.InputType['KeystoresAliasesSelfSignedCertSubjectAlternativeDnsNamesArgs']] subject_alternative_dns_names: List of alternative host names. Maximum length is 255 characters for each value.
               Structure is documented below.
        :param pulumi.Input[str] type: Optional.Type of Alias
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _KeystoresAliasesSelfSignedCertState.__new__(_KeystoresAliasesSelfSignedCertState)

        __props__.__dict__["alias"] = alias
        __props__.__dict__["cert_validity_in_days"] = cert_validity_in_days
        __props__.__dict__["certs_infos"] = certs_infos
        __props__.__dict__["environment"] = environment
        __props__.__dict__["key_size"] = key_size
        __props__.__dict__["keystore"] = keystore
        __props__.__dict__["org_id"] = org_id
        __props__.__dict__["sig_alg"] = sig_alg
        __props__.__dict__["subject"] = subject
        __props__.__dict__["subject_alternative_dns_names"] = subject_alternative_dns_names
        __props__.__dict__["type"] = type
        return KeystoresAliasesSelfSignedCert(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter
    def alias(self) -> pulumi.Output[str]:
        """
        Alias for the key/certificate pair. Values must match the regular expression [\\w\\s-.]{1,255}.
        This must be provided for all formats except selfsignedcert; self-signed certs may specify the alias in either
        this parameter or the JSON body.
        """
        return pulumi.get(self, "alias")

    @property
    @pulumi.getter(name="certValidityInDays")
    def cert_validity_in_days(self) -> pulumi.Output[Optional[int]]:
        """
        Validity duration of certificate, in days. Accepts positive non-zero value. Defaults to 365.
        """
        return pulumi.get(self, "cert_validity_in_days")

    @property
    @pulumi.getter(name="certsInfos")
    def certs_infos(self) -> pulumi.Output[Sequence['outputs.KeystoresAliasesSelfSignedCertCertsInfo']]:
        """
        Chain of certificates under this alias.
        Structure is documented below.
        """
        return pulumi.get(self, "certs_infos")

    @property
    @pulumi.getter
    def environment(self) -> pulumi.Output[str]:
        """
        The Apigee environment name
        """
        return pulumi.get(self, "environment")

    @property
    @pulumi.getter(name="keySize")
    def key_size(self) -> pulumi.Output[Optional[str]]:
        """
        Key size. Default and maximum value is 2048 bits.
        """
        return pulumi.get(self, "key_size")

    @property
    @pulumi.getter
    def keystore(self) -> pulumi.Output[str]:
        """
        The Apigee keystore name associated in an Apigee environment
        """
        return pulumi.get(self, "keystore")

    @property
    @pulumi.getter(name="orgId")
    def org_id(self) -> pulumi.Output[str]:
        """
        The Apigee Organization name associated with the Apigee environment
        """
        return pulumi.get(self, "org_id")

    @property
    @pulumi.getter(name="sigAlg")
    def sig_alg(self) -> pulumi.Output[str]:
        """
        Signature algorithm to generate private key. Valid values are SHA512withRSA, SHA384withRSA, and SHA256withRSA
        """
        return pulumi.get(self, "sig_alg")

    @property
    @pulumi.getter
    def subject(self) -> pulumi.Output['outputs.KeystoresAliasesSelfSignedCertSubject']:
        """
        Subject details.
        Structure is documented below.
        """
        return pulumi.get(self, "subject")

    @property
    @pulumi.getter(name="subjectAlternativeDnsNames")
    def subject_alternative_dns_names(self) -> pulumi.Output[Optional['outputs.KeystoresAliasesSelfSignedCertSubjectAlternativeDnsNames']]:
        """
        List of alternative host names. Maximum length is 255 characters for each value.
        Structure is documented below.
        """
        return pulumi.get(self, "subject_alternative_dns_names")

    @property
    @pulumi.getter
    def type(self) -> pulumi.Output[str]:
        """
        Optional.Type of Alias
        """
        return pulumi.get(self, "type")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from . import outputs
from ._inputs import *

__all__ = ['PrivateCloudArgs', 'PrivateCloud']

@pulumi.input_type
class PrivateCloudArgs:
    def __init__(__self__, *,
                 location: pulumi.Input[str],
                 management_cluster: pulumi.Input['PrivateCloudManagementClusterArgs'],
                 network_config: pulumi.Input['PrivateCloudNetworkConfigArgs'],
                 description: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 project: Optional[pulumi.Input[str]] = None):
        """
        The set of arguments for constructing a PrivateCloud resource.
        :param pulumi.Input[str] location: The location where the PrivateCloud should reside.
        :param pulumi.Input['PrivateCloudManagementClusterArgs'] management_cluster: The management cluster for this private cloud. This used for creating and managing the default cluster.
               Structure is documented below.
        :param pulumi.Input['PrivateCloudNetworkConfigArgs'] network_config: Network configuration in the consumer project with which the peering has to be done.
               Structure is documented below.
        :param pulumi.Input[str] description: User-provided description for this private cloud.
        :param pulumi.Input[str] name: The ID of the PrivateCloud.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        """
        pulumi.set(__self__, "location", location)
        pulumi.set(__self__, "management_cluster", management_cluster)
        pulumi.set(__self__, "network_config", network_config)
        if description is not None:
            pulumi.set(__self__, "description", description)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if project is not None:
            pulumi.set(__self__, "project", project)

    @property
    @pulumi.getter
    def location(self) -> pulumi.Input[str]:
        """
        The location where the PrivateCloud should reside.
        """
        return pulumi.get(self, "location")

    @location.setter
    def location(self, value: pulumi.Input[str]):
        pulumi.set(self, "location", value)

    @property
    @pulumi.getter(name="managementCluster")
    def management_cluster(self) -> pulumi.Input['PrivateCloudManagementClusterArgs']:
        """
        The management cluster for this private cloud. This used for creating and managing the default cluster.
        Structure is documented below.
        """
        return pulumi.get(self, "management_cluster")

    @management_cluster.setter
    def management_cluster(self, value: pulumi.Input['PrivateCloudManagementClusterArgs']):
        pulumi.set(self, "management_cluster", value)

    @property
    @pulumi.getter(name="networkConfig")
    def network_config(self) -> pulumi.Input['PrivateCloudNetworkConfigArgs']:
        """
        Network configuration in the consumer project with which the peering has to be done.
        Structure is documented below.
        """
        return pulumi.get(self, "network_config")

    @network_config.setter
    def network_config(self, value: pulumi.Input['PrivateCloudNetworkConfigArgs']):
        pulumi.set(self, "network_config", value)

    @property
    @pulumi.getter
    def description(self) -> Optional[pulumi.Input[str]]:
        """
        User-provided description for this private cloud.
        """
        return pulumi.get(self, "description")

    @description.setter
    def description(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "description", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[str]]:
        """
        The ID of the PrivateCloud.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter
    def project(self) -> Optional[pulumi.Input[str]]:
        """
        The ID of the project in which the resource belongs.
        If it is not provided, the provider project is used.
        """
        return pulumi.get(self, "project")

    @project.setter
    def project(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "project", value)


@pulumi.input_type
class _PrivateCloudState:
    def __init__(__self__, *,
                 description: Optional[pulumi.Input[str]] = None,
                 hcxes: Optional[pulumi.Input[Sequence[pulumi.Input['PrivateCloudHcxArgs']]]] = None,
                 location: Optional[pulumi.Input[str]] = None,
                 management_cluster: Optional[pulumi.Input['PrivateCloudManagementClusterArgs']] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 network_config: Optional[pulumi.Input['PrivateCloudNetworkConfigArgs']] = None,
                 nsxes: Optional[pulumi.Input[Sequence[pulumi.Input['PrivateCloudNsxArgs']]]] = None,
                 project: Optional[pulumi.Input[str]] = None,
                 state: Optional[pulumi.Input[str]] = None,
                 uid: Optional[pulumi.Input[str]] = None,
                 vcenters: Optional[pulumi.Input[Sequence[pulumi.Input['PrivateCloudVcenterArgs']]]] = None):
        """
        Input properties used for looking up and filtering PrivateCloud resources.
        :param pulumi.Input[str] description: User-provided description for this private cloud.
        :param pulumi.Input[Sequence[pulumi.Input['PrivateCloudHcxArgs']]] hcxes: Details about a HCX Cloud Manager appliance.
               Structure is documented below.
        :param pulumi.Input[str] location: The location where the PrivateCloud should reside.
        :param pulumi.Input['PrivateCloudManagementClusterArgs'] management_cluster: The management cluster for this private cloud. This used for creating and managing the default cluster.
               Structure is documented below.
        :param pulumi.Input[str] name: The ID of the PrivateCloud.
        :param pulumi.Input['PrivateCloudNetworkConfigArgs'] network_config: Network configuration in the consumer project with which the peering has to be done.
               Structure is documented below.
        :param pulumi.Input[Sequence[pulumi.Input['PrivateCloudNsxArgs']]] nsxes: Details about a NSX Manager appliance.
               Structure is documented below.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[str] state: State of the appliance.
               Possible values are: `ACTIVE`, `CREATING`.
        :param pulumi.Input[str] uid: System-generated unique identifier for the resource.
        :param pulumi.Input[Sequence[pulumi.Input['PrivateCloudVcenterArgs']]] vcenters: Details about a vCenter Server management appliance.
               Structure is documented below.
        """
        if description is not None:
            pulumi.set(__self__, "description", description)
        if hcxes is not None:
            pulumi.set(__self__, "hcxes", hcxes)
        if location is not None:
            pulumi.set(__self__, "location", location)
        if management_cluster is not None:
            pulumi.set(__self__, "management_cluster", management_cluster)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if network_config is not None:
            pulumi.set(__self__, "network_config", network_config)
        if nsxes is not None:
            pulumi.set(__self__, "nsxes", nsxes)
        if project is not None:
            pulumi.set(__self__, "project", project)
        if state is not None:
            pulumi.set(__self__, "state", state)
        if uid is not None:
            pulumi.set(__self__, "uid", uid)
        if vcenters is not None:
            pulumi.set(__self__, "vcenters", vcenters)

    @property
    @pulumi.getter
    def description(self) -> Optional[pulumi.Input[str]]:
        """
        User-provided description for this private cloud.
        """
        return pulumi.get(self, "description")

    @description.setter
    def description(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "description", value)

    @property
    @pulumi.getter
    def hcxes(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['PrivateCloudHcxArgs']]]]:
        """
        Details about a HCX Cloud Manager appliance.
        Structure is documented below.
        """
        return pulumi.get(self, "hcxes")

    @hcxes.setter
    def hcxes(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['PrivateCloudHcxArgs']]]]):
        pulumi.set(self, "hcxes", value)

    @property
    @pulumi.getter
    def location(self) -> Optional[pulumi.Input[str]]:
        """
        The location where the PrivateCloud should reside.
        """
        return pulumi.get(self, "location")

    @location.setter
    def location(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "location", value)

    @property
    @pulumi.getter(name="managementCluster")
    def management_cluster(self) -> Optional[pulumi.Input['PrivateCloudManagementClusterArgs']]:
        """
        The management cluster for this private cloud. This used for creating and managing the default cluster.
        Structure is documented below.
        """
        return pulumi.get(self, "management_cluster")

    @management_cluster.setter
    def management_cluster(self, value: Optional[pulumi.Input['PrivateCloudManagementClusterArgs']]):
        pulumi.set(self, "management_cluster", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[str]]:
        """
        The ID of the PrivateCloud.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter(name="networkConfig")
    def network_config(self) -> Optional[pulumi.Input['PrivateCloudNetworkConfigArgs']]:
        """
        Network configuration in the consumer project with which the peering has to be done.
        Structure is documented below.
        """
        return pulumi.get(self, "network_config")

    @network_config.setter
    def network_config(self, value: Optional[pulumi.Input['PrivateCloudNetworkConfigArgs']]):
        pulumi.set(self, "network_config", value)

    @property
    @pulumi.getter
    def nsxes(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['PrivateCloudNsxArgs']]]]:
        """
        Details about a NSX Manager appliance.
        Structure is documented below.
        """
        return pulumi.get(self, "nsxes")

    @nsxes.setter
    def nsxes(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['PrivateCloudNsxArgs']]]]):
        pulumi.set(self, "nsxes", value)

    @property
    @pulumi.getter
    def project(self) -> Optional[pulumi.Input[str]]:
        """
        The ID of the project in which the resource belongs.
        If it is not provided, the provider project is used.
        """
        return pulumi.get(self, "project")

    @project.setter
    def project(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "project", value)

    @property
    @pulumi.getter
    def state(self) -> Optional[pulumi.Input[str]]:
        """
        State of the appliance.
        Possible values are: `ACTIVE`, `CREATING`.
        """
        return pulumi.get(self, "state")

    @state.setter
    def state(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "state", value)

    @property
    @pulumi.getter
    def uid(self) -> Optional[pulumi.Input[str]]:
        """
        System-generated unique identifier for the resource.
        """
        return pulumi.get(self, "uid")

    @uid.setter
    def uid(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "uid", value)

    @property
    @pulumi.getter
    def vcenters(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['PrivateCloudVcenterArgs']]]]:
        """
        Details about a vCenter Server management appliance.
        Structure is documented below.
        """
        return pulumi.get(self, "vcenters")

    @vcenters.setter
    def vcenters(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['PrivateCloudVcenterArgs']]]]):
        pulumi.set(self, "vcenters", value)


class PrivateCloud(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 description: Optional[pulumi.Input[str]] = None,
                 location: Optional[pulumi.Input[str]] = None,
                 management_cluster: Optional[pulumi.Input[pulumi.InputType['PrivateCloudManagementClusterArgs']]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 network_config: Optional[pulumi.Input[pulumi.InputType['PrivateCloudNetworkConfigArgs']]] = None,
                 project: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        """
        ## Example Usage
        ### Vmware Engine Private Cloud Basic

        ```python
        import pulumi
        import pulumi_gcp as gcp

        pc_nw = gcp.vmwareengine.Network("pc-nw",
            location="us-west1",
            type="LEGACY",
            description="PC network description.",
            opts=pulumi.ResourceOptions(provider=google_beta))
        vmw_engine_pc = gcp.vmwareengine.PrivateCloud("vmw-engine-pc",
            location="us-west1-a",
            description="Sample test PC.",
            network_config=gcp.vmwareengine.PrivateCloudNetworkConfigArgs(
                management_cidr="192.168.30.0/24",
                vmware_engine_network=pc_nw.id,
            ),
            management_cluster=gcp.vmwareengine.PrivateCloudManagementClusterArgs(
                cluster_id="sample-mgmt-cluster",
                node_type_configs=[gcp.vmwareengine.PrivateCloudManagementClusterNodeTypeConfigArgs(
                    node_type_id="standard-72",
                    node_count=3,
                )],
            ),
            opts=pulumi.ResourceOptions(provider=google_beta))
        ```
        ### Vmware Engine Private Cloud Full

        ```python
        import pulumi
        import pulumi_gcp as gcp

        pc_nw = gcp.vmwareengine.Network("pc-nw",
            location="us-west1",
            type="LEGACY",
            description="PC network description.",
            opts=pulumi.ResourceOptions(provider=google_beta))
        vmw_engine_pc = gcp.vmwareengine.PrivateCloud("vmw-engine-pc",
            location="us-west1-a",
            description="Sample test PC.",
            network_config=gcp.vmwareengine.PrivateCloudNetworkConfigArgs(
                management_cidr="192.168.30.0/24",
                vmware_engine_network=pc_nw.id,
            ),
            management_cluster=gcp.vmwareengine.PrivateCloudManagementClusterArgs(
                cluster_id="sample-mgmt-cluster",
                node_type_configs=[gcp.vmwareengine.PrivateCloudManagementClusterNodeTypeConfigArgs(
                    node_type_id="standard-72",
                    node_count=3,
                    custom_core_count=32,
                )],
            ),
            opts=pulumi.ResourceOptions(provider=google_beta))
        ```

        ## Import

        PrivateCloud can be imported using any of these accepted formats

        ```sh
         $ pulumi import gcp:vmwareengine/privateCloud:PrivateCloud default projects/{{project}}/locations/{{location}}/privateClouds/{{name}}
        ```

        ```sh
         $ pulumi import gcp:vmwareengine/privateCloud:PrivateCloud default {{project}}/{{location}}/{{name}}
        ```

        ```sh
         $ pulumi import gcp:vmwareengine/privateCloud:PrivateCloud default {{location}}/{{name}}
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] description: User-provided description for this private cloud.
        :param pulumi.Input[str] location: The location where the PrivateCloud should reside.
        :param pulumi.Input[pulumi.InputType['PrivateCloudManagementClusterArgs']] management_cluster: The management cluster for this private cloud. This used for creating and managing the default cluster.
               Structure is documented below.
        :param pulumi.Input[str] name: The ID of the PrivateCloud.
        :param pulumi.Input[pulumi.InputType['PrivateCloudNetworkConfigArgs']] network_config: Network configuration in the consumer project with which the peering has to be done.
               Structure is documented below.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: PrivateCloudArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        ## Example Usage
        ### Vmware Engine Private Cloud Basic

        ```python
        import pulumi
        import pulumi_gcp as gcp

        pc_nw = gcp.vmwareengine.Network("pc-nw",
            location="us-west1",
            type="LEGACY",
            description="PC network description.",
            opts=pulumi.ResourceOptions(provider=google_beta))
        vmw_engine_pc = gcp.vmwareengine.PrivateCloud("vmw-engine-pc",
            location="us-west1-a",
            description="Sample test PC.",
            network_config=gcp.vmwareengine.PrivateCloudNetworkConfigArgs(
                management_cidr="192.168.30.0/24",
                vmware_engine_network=pc_nw.id,
            ),
            management_cluster=gcp.vmwareengine.PrivateCloudManagementClusterArgs(
                cluster_id="sample-mgmt-cluster",
                node_type_configs=[gcp.vmwareengine.PrivateCloudManagementClusterNodeTypeConfigArgs(
                    node_type_id="standard-72",
                    node_count=3,
                )],
            ),
            opts=pulumi.ResourceOptions(provider=google_beta))
        ```
        ### Vmware Engine Private Cloud Full

        ```python
        import pulumi
        import pulumi_gcp as gcp

        pc_nw = gcp.vmwareengine.Network("pc-nw",
            location="us-west1",
            type="LEGACY",
            description="PC network description.",
            opts=pulumi.ResourceOptions(provider=google_beta))
        vmw_engine_pc = gcp.vmwareengine.PrivateCloud("vmw-engine-pc",
            location="us-west1-a",
            description="Sample test PC.",
            network_config=gcp.vmwareengine.PrivateCloudNetworkConfigArgs(
                management_cidr="192.168.30.0/24",
                vmware_engine_network=pc_nw.id,
            ),
            management_cluster=gcp.vmwareengine.PrivateCloudManagementClusterArgs(
                cluster_id="sample-mgmt-cluster",
                node_type_configs=[gcp.vmwareengine.PrivateCloudManagementClusterNodeTypeConfigArgs(
                    node_type_id="standard-72",
                    node_count=3,
                    custom_core_count=32,
                )],
            ),
            opts=pulumi.ResourceOptions(provider=google_beta))
        ```

        ## Import

        PrivateCloud can be imported using any of these accepted formats

        ```sh
         $ pulumi import gcp:vmwareengine/privateCloud:PrivateCloud default projects/{{project}}/locations/{{location}}/privateClouds/{{name}}
        ```

        ```sh
         $ pulumi import gcp:vmwareengine/privateCloud:PrivateCloud default {{project}}/{{location}}/{{name}}
        ```

        ```sh
         $ pulumi import gcp:vmwareengine/privateCloud:PrivateCloud default {{location}}/{{name}}
        ```

        :param str resource_name: The name of the resource.
        :param PrivateCloudArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(PrivateCloudArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 description: Optional[pulumi.Input[str]] = None,
                 location: Optional[pulumi.Input[str]] = None,
                 management_cluster: Optional[pulumi.Input[pulumi.InputType['PrivateCloudManagementClusterArgs']]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 network_config: Optional[pulumi.Input[pulumi.InputType['PrivateCloudNetworkConfigArgs']]] = None,
                 project: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = PrivateCloudArgs.__new__(PrivateCloudArgs)

            __props__.__dict__["description"] = description
            if location is None and not opts.urn:
                raise TypeError("Missing required property 'location'")
            __props__.__dict__["location"] = location
            if management_cluster is None and not opts.urn:
                raise TypeError("Missing required property 'management_cluster'")
            __props__.__dict__["management_cluster"] = management_cluster
            __props__.__dict__["name"] = name
            if network_config is None and not opts.urn:
                raise TypeError("Missing required property 'network_config'")
            __props__.__dict__["network_config"] = network_config
            __props__.__dict__["project"] = project
            __props__.__dict__["hcxes"] = None
            __props__.__dict__["nsxes"] = None
            __props__.__dict__["state"] = None
            __props__.__dict__["uid"] = None
            __props__.__dict__["vcenters"] = None
        super(PrivateCloud, __self__).__init__(
            'gcp:vmwareengine/privateCloud:PrivateCloud',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            description: Optional[pulumi.Input[str]] = None,
            hcxes: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['PrivateCloudHcxArgs']]]]] = None,
            location: Optional[pulumi.Input[str]] = None,
            management_cluster: Optional[pulumi.Input[pulumi.InputType['PrivateCloudManagementClusterArgs']]] = None,
            name: Optional[pulumi.Input[str]] = None,
            network_config: Optional[pulumi.Input[pulumi.InputType['PrivateCloudNetworkConfigArgs']]] = None,
            nsxes: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['PrivateCloudNsxArgs']]]]] = None,
            project: Optional[pulumi.Input[str]] = None,
            state: Optional[pulumi.Input[str]] = None,
            uid: Optional[pulumi.Input[str]] = None,
            vcenters: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['PrivateCloudVcenterArgs']]]]] = None) -> 'PrivateCloud':
        """
        Get an existing PrivateCloud resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] description: User-provided description for this private cloud.
        :param pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['PrivateCloudHcxArgs']]]] hcxes: Details about a HCX Cloud Manager appliance.
               Structure is documented below.
        :param pulumi.Input[str] location: The location where the PrivateCloud should reside.
        :param pulumi.Input[pulumi.InputType['PrivateCloudManagementClusterArgs']] management_cluster: The management cluster for this private cloud. This used for creating and managing the default cluster.
               Structure is documented below.
        :param pulumi.Input[str] name: The ID of the PrivateCloud.
        :param pulumi.Input[pulumi.InputType['PrivateCloudNetworkConfigArgs']] network_config: Network configuration in the consumer project with which the peering has to be done.
               Structure is documented below.
        :param pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['PrivateCloudNsxArgs']]]] nsxes: Details about a NSX Manager appliance.
               Structure is documented below.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[str] state: State of the appliance.
               Possible values are: `ACTIVE`, `CREATING`.
        :param pulumi.Input[str] uid: System-generated unique identifier for the resource.
        :param pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['PrivateCloudVcenterArgs']]]] vcenters: Details about a vCenter Server management appliance.
               Structure is documented below.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _PrivateCloudState.__new__(_PrivateCloudState)

        __props__.__dict__["description"] = description
        __props__.__dict__["hcxes"] = hcxes
        __props__.__dict__["location"] = location
        __props__.__dict__["management_cluster"] = management_cluster
        __props__.__dict__["name"] = name
        __props__.__dict__["network_config"] = network_config
        __props__.__dict__["nsxes"] = nsxes
        __props__.__dict__["project"] = project
        __props__.__dict__["state"] = state
        __props__.__dict__["uid"] = uid
        __props__.__dict__["vcenters"] = vcenters
        return PrivateCloud(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter
    def description(self) -> pulumi.Output[Optional[str]]:
        """
        User-provided description for this private cloud.
        """
        return pulumi.get(self, "description")

    @property
    @pulumi.getter
    def hcxes(self) -> pulumi.Output[Sequence['outputs.PrivateCloudHcx']]:
        """
        Details about a HCX Cloud Manager appliance.
        Structure is documented below.
        """
        return pulumi.get(self, "hcxes")

    @property
    @pulumi.getter
    def location(self) -> pulumi.Output[str]:
        """
        The location where the PrivateCloud should reside.
        """
        return pulumi.get(self, "location")

    @property
    @pulumi.getter(name="managementCluster")
    def management_cluster(self) -> pulumi.Output['outputs.PrivateCloudManagementCluster']:
        """
        The management cluster for this private cloud. This used for creating and managing the default cluster.
        Structure is documented below.
        """
        return pulumi.get(self, "management_cluster")

    @property
    @pulumi.getter
    def name(self) -> pulumi.Output[str]:
        """
        The ID of the PrivateCloud.
        """
        return pulumi.get(self, "name")

    @property
    @pulumi.getter(name="networkConfig")
    def network_config(self) -> pulumi.Output['outputs.PrivateCloudNetworkConfig']:
        """
        Network configuration in the consumer project with which the peering has to be done.
        Structure is documented below.
        """
        return pulumi.get(self, "network_config")

    @property
    @pulumi.getter
    def nsxes(self) -> pulumi.Output[Sequence['outputs.PrivateCloudNsx']]:
        """
        Details about a NSX Manager appliance.
        Structure is documented below.
        """
        return pulumi.get(self, "nsxes")

    @property
    @pulumi.getter
    def project(self) -> pulumi.Output[str]:
        """
        The ID of the project in which the resource belongs.
        If it is not provided, the provider project is used.
        """
        return pulumi.get(self, "project")

    @property
    @pulumi.getter
    def state(self) -> pulumi.Output[str]:
        """
        State of the appliance.
        Possible values are: `ACTIVE`, `CREATING`.
        """
        return pulumi.get(self, "state")

    @property
    @pulumi.getter
    def uid(self) -> pulumi.Output[str]:
        """
        System-generated unique identifier for the resource.
        """
        return pulumi.get(self, "uid")

    @property
    @pulumi.getter
    def vcenters(self) -> pulumi.Output[Sequence['outputs.PrivateCloudVcenter']]:
        """
        Details about a vCenter Server management appliance.
        Structure is documented below.
        """
        return pulumi.get(self, "vcenters")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities

__all__ = [
    'GetKeyRingIamPolicyResult',
    'AwaitableGetKeyRingIamPolicyResult',
    'get_key_ring_iam_policy',
    'get_key_ring_iam_policy_output',
]

@pulumi.output_type
class GetKeyRingIamPolicyResult:
    """
    A collection of values returned by getKeyRingIamPolicy.
    """
    def __init__(__self__, etag=None, id=None, key_ring_id=None, policy_data=None):
        if etag and not isinstance(etag, str):
            raise TypeError("Expected argument 'etag' to be a str")
        pulumi.set(__self__, "etag", etag)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if key_ring_id and not isinstance(key_ring_id, str):
            raise TypeError("Expected argument 'key_ring_id' to be a str")
        pulumi.set(__self__, "key_ring_id", key_ring_id)
        if policy_data and not isinstance(policy_data, str):
            raise TypeError("Expected argument 'policy_data' to be a str")
        pulumi.set(__self__, "policy_data", policy_data)

    @property
    @pulumi.getter
    def etag(self) -> str:
        """
        (Computed) The etag of the IAM policy.
        """
        return pulumi.get(self, "etag")

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="keyRingId")
    def key_ring_id(self) -> str:
        return pulumi.get(self, "key_ring_id")

    @property
    @pulumi.getter(name="policyData")
    def policy_data(self) -> str:
        """
        (Computed) The policy data
        """
        return pulumi.get(self, "policy_data")


class AwaitableGetKeyRingIamPolicyResult(GetKeyRingIamPolicyResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetKeyRingIamPolicyResult(
            etag=self.etag,
            id=self.id,
            key_ring_id=self.key_ring_id,
            policy_data=self.policy_data)


def get_key_ring_iam_policy(key_ring_id: Optional[str] = None,
                            opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetKeyRingIamPolicyResult:
    """
    Retrieves the current IAM policy data for a Google Cloud KMS key ring.

    ## example

    ```python
    import pulumi
    import pulumi_gcp as gcp

    test_key_ring_iam_policy = gcp.kms.get_key_ring_iam_policy(key_ring_id="{project_id}/{location_name}/{key_ring_name}")
    ```


    :param str key_ring_id: The key ring ID, in the form
           `{project_id}/{location_name}/{key_ring_name}` or
           `{location_name}/{key_ring_name}`. In the second form, the provider's
           project setting will be used as a fallback.
    """
    __args__ = dict()
    __args__['keyRingId'] = key_ring_id
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('gcp:kms/getKeyRingIamPolicy:getKeyRingIamPolicy', __args__, opts=opts, typ=GetKeyRingIamPolicyResult).value

    return AwaitableGetKeyRingIamPolicyResult(
        etag=pulumi.get(__ret__, 'etag'),
        id=pulumi.get(__ret__, 'id'),
        key_ring_id=pulumi.get(__ret__, 'key_ring_id'),
        policy_data=pulumi.get(__ret__, 'policy_data'))


@_utilities.lift_output_func(get_key_ring_iam_policy)
def get_key_ring_iam_policy_output(key_ring_id: Optional[pulumi.Input[str]] = None,
                                   opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[GetKeyRingIamPolicyResult]:
    """
    Retrieves the current IAM policy data for a Google Cloud KMS key ring.

    ## example

    ```python
    import pulumi
    import pulumi_gcp as gcp

    test_key_ring_iam_policy = gcp.kms.get_key_ring_iam_policy(key_ring_id="{project_id}/{location_name}/{key_ring_name}")
    ```


    :param str key_ring_id: The key ring ID, in the form
           `{project_id}/{location_name}/{key_ring_name}` or
           `{location_name}/{key_ring_name}`. In the second form, the provider's
           project setting will be used as a fallback.
    """
    ...
